#!/usr/bin/env python3
# © Copyright 2020-2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates and licensors
# (“Micro Focus”) are as may be set forth in the express warranty statements accompanying such
# products and services. Nothing herein should be construed as constituting an additional
# warranty. Micro Focus shall not be liable for technical or editorial errors or omissions
# contained herein. The information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential information
# and a valid license is required for possession, use or copying. If this work is provided to the
# U.S. Government, consistent with FAR 12.211 and 12.212, Commercial Computer Software,
# Computer Software Documentation, and Technical Data for Commercial Items are licensed
# to the U.S. Government under vendor's standard commercial license.

"""
Python script to fetch the health status of SMS for OES dashboard.
"""

import json
import subprocess
import sys
from collections import OrderedDict
from datetime import datetime

"""
service - Name of the service
properties - Systemctl properties
daemons - List of daemons. Any new daemon should be added here.
result - Final JSON of SMS health status.
tasks - Threshold - Status will become Partially Healthy when no of tasks are lesser than this.
healthy/unhealthy/partially_healthy - Status of service
"""

service = "novell-smdrd"
properties = ["ActiveState", "TasksCurrent", "ActiveEnterTimestamp", "InactiveEnterTimestamp"]
daemons = ["smdrd"]
result = OrderedDict([
    ("serviceName", "OES Backup / Storage Managament Services (SMS)"),
    ("status", "")])
tasks = 3
healthy = "Healthy"
unhealthy = "Unhealthy"
partially_healthy = "Partially Healthy"

# The dict 'additional_info_key' contains the keys of resource bundle. The below 'keys' (ex: Err_Sms_DaemonDown)
# will be used by dashboard app to get the localised string.
additional_info_key = { "daemon_down": "Err_Sms_DaemonDown", "service_down": "Common_ServiceInactive", "loaded_tsa" : "Info_Sms_LoadedTsa", "service_restart": "Common_ResolveService", "resolve_daemon": "Common_ResolveDaemon" }

def get_systemctl_properties():
    """
    Function to run systemctl and capture the output.
    :return: dict: Result of systemctl
    """

    systemctl_result = dict()
    cmd = "systemctl show " + service + " --property=" + ",".join(properties)
    output = run_command(cmd)

    for line in output.splitlines():
        key, value = line.strip().split("=")
        systemctl_result[key] = value

    return systemctl_result


def is_daemon_running(process):
    """
    Function to check if the daemon process is running or not.
    :param process: Name of process
    :return: boolean: Status of process
    """

    cmd = "ps -ef | awk '$3 == 1' | grep -w '" + process + "' | wc -l"
    status = run_command(cmd)
    if int(status) > 0:
        return True
    else:
        return False


def generate_json(data):
    """
    Takes dict and converts to json.
    :param data: Data in dictionary form.
    :return: json: Converted value
    """

    formatted_data = json.dumps(data)
    return formatted_data


def run_command(cmd):
    """
    Function to execute linux command and capture the result
    :param cmd: linux command
    :return: Output of the command
    """

    err, output = None, None
    ret = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    try:
        (output, err) = ret.communicate(timeout=5)
    except TimeoutError:
        ret.kill()

    if err:
        sys.exit(2)
    elif output:
        return output.decode('utf-8')


def add_tsa_list():
    """
    Method to add loaded TSAs to result.
    """

    cmd = "/opt/novell/sms/bin/smsconfig -t | tail -n +2"
    output = run_command(cmd)
    if output:
        tsa = ",".join(output.split())
        info = additional_info_key["loaded_tsa"] + "," + tsa
        result.update({"AdditionalInfo" :  info.split(",")})


def update_result(state, uptime=0, info="NA"):
    """
    Function to update the result dictionary.
    :param state: State of the service (healthy/partially_healthy/unhealthy)
    :param uptime: Uptime of the service
    :param info: Information for Additional-Info
    """

    result["status"] = state
    if state == healthy:
        result.update({"runningSince" : uptime})
        add_tsa_list()
    elif state == partially_healthy:
        result.update({"runningSince" : uptime})
        result.update({"AdditionalInfo" : info.split(",")})
    elif state == unhealthy:
        result.update({"AdditionalInfo" : info.split(",")})


def daemons_status():
    """
    Inspects if all the daemons are running.
    :return: boolean: status of daemon
    """

    status = True
    for daemon in daemons:
        status = is_daemon_running(daemon)
        if status == -1:
            sys.exit(2)
        elif not status:
            break
    return status


def check_tasks(service_properties):
    """
    Function to update status based on the number of tasks running.
    :param service_properties: Dictionary of service properties
    """

    task = service_properties.get("TasksCurrent")
    timestamp = service_properties.get("ActiveEnterTimestamp")
    if timestamp:
        if timestamp =="n/a":
            uptime = 0
        else:
            uptime = int(datetime.strptime(timestamp, "%a %Y-%m-%d %H:%M:%S %Z").timestamp())
    if int(task) < tasks:
        info = additional_info_key["service_restart"]
        update_result(partially_healthy, uptime, info)
    else:
        update_result(healthy, uptime)


def get_sms_health():
    """
    The main Function to get the status of SMS. Here it checks novell-sms service,
    related daemons, uptime and number of tasks SMS is running to decide the health.

    prints json object: SMS health
    :return 0 on success
    """

    service_properties = get_systemctl_properties()
    if service_properties != -1:
        state = service_properties.get("ActiveState")
        if state == "active":
            if not daemons_status():
                daemon_down = additional_info_key["daemon_down"] + "," + additional_info_key["resolve_daemon"]
                update_result(unhealthy, info=daemon_down)
            else:
                check_tasks(service_properties)
        else:
            service_down = "SMDRD," + additional_info_key["service_down"] + "," + additional_info_key["service_restart"]
            update_result(unhealthy, info=service_down)
    else:
        sys.exit(2)

    print(generate_json(result))
    return 0


# Script starts from here
if __name__ == '__main__':
    get_sms_health()
